"""
Astral provides the 'mavro/distbuilder/astral' directory that Mavro uses to build files into executable format.
Astral is a .py->.exe distribution builder that uses the PyInstaller and UPX. Astral, not including PyInstaller or UPX,
is licensed by the MavroLang development foundation. Astral may only be installed for free as part of MavroLang,
or purchased separately with a separate license from Astral distributors.

PyInstaller license: https://pyinstaller.org/en/stable/license.html
UPX license: https://github.com/upx/upx/blob/devel/LICENSE
"""